import os
import subprocess
import CsoundAPI

from time import sleep

class CsoundXOAgent():

    def __init__(self, host, port):
        self.host = host
        self.port = port
        self.status = 0

        if host == "localhost":
            if not self.launchServer():
                print "Error in launching server."
                self.status = -1
                return
        
        if not self.loadOrc():
            print "Error in reading orchestra or score file."
            self.status = -2
            return

        sleep(1)        

        if not self.connectCsound():
            print "Error in connecting to csound server."
            self.status = -3
            return

        if not self.runCsound():
            print "Error in setting up csound server."
            self.status = -4
            return
        
        self.status = 1        

    def launchServer(self):
        try:
            self.server = subprocess.Popen("./csoundxo")            
        except:
            return False
        return True
        
    def connectCsound(self):
        try:
            self.connect(self.host, self.port)
        except:
            return False
        return True
        
    def runCsound(self):
        try:
            self.setup()
            self.perform()
        except:
            return False
        return True

    def loadOrc(self):       
        try:
            forc = open("./mpainter.orc","r")
            fsco = open("./mpainter.sco","r")
            self.orchestra = forc.read()
            self.score = fsco.read()
            forc.close()
            fsco.close()
        except:
            return False
        return True
        
    def connect(self, address, port):
        self.csound = CsoundAPI.CsoundAPI(address, port)
        sleep(1)

    def setup(self):
        self.csound.newInstance()
        self.csound.sendOrcStr(self.orchestra)
        self.csound.sendScoStr(self.score)
        self.csound.setDisplays(0)
        self.csound.setMsgLevel(255)
        self.csound.setIOBufFrames(1024)
        self.csound.sendPrep()
#        self.csound.setP2Timer(0.1, callback)
#        self.csound.enterBeatInputMode()

    def perform(self):    
        self.csound.sendPlay()
        
    def sendClearLines(self):
        if self.status == 1:
            self.csound.sendClearLines()
    
    def sendLinevt(self, eventStr):
        if self.status == 1:
            self.csound.sendLinevt(eventStr)
            
    def current_kperiod(self):
        if self.status == 1:
            return self.csound.current_kperiod()
        else:
            return -1
        
    def close(self):
        if self.status == 1:
            self.csound.sendPause()
            self.csound.serverShutdown()
#            self.csound.close()
        self.status = 0
