import gtk
import math
import cairo

_showDebugMsg = False

class ToolbarArea(gtk.DrawingArea):

    def __init__(self, wid, hei, mp):

        self.init_data()
        self.init_graphics()

        gtk.DrawingArea.__init__(self)

        self.width, self.height = wid, hei
        self.set_size_request(self.width, self.height)

        # Event signals
        self.connect("expose_event", self.expose_event)
        self.connect("configure_event", self.configure_event)
        self.connect("leave_notify_event", self.leave_notify_event)
        self.connect("motion_notify_event", self.motion_notify_event)
        self.connect("button_press_event", self.button_press_event)
        self.connect("button_release_event", self.button_release_event)

        self.set_events(gtk.gdk.EXPOSURE_MASK
                        | gtk.gdk.LEAVE_NOTIFY_MASK
                        | gtk.gdk.BUTTON_PRESS_MASK
                        | gtk.gdk.BUTTON_RELEASE_MASK
                        | gtk.gdk.POINTER_MOTION_MASK
                        | gtk.gdk.POINTER_MOTION_HINT_MASK)

        self.main = mp

    def init_data(self):	
        self.toolbarw = 36        
        self.cx = 92
        self.cy = 12
        self.load_icon = 0
        self.toolsel = 2
        self.colorsel = 0
        self.is_dragging = 0

        self.fix_button = 0	
        
        self.sx = self.cx + self.toolbarw*14 + 40   # slider
        self.sy = self.cy + self.toolbarw/2 - 2
        self.sw = 160
        self.sval = 0.5
        
        self.pstate = 0
        self.fstate = 0
        
    def get_color_attr(self, c):
        if c == 0:   # red
            return (1, 0.225, 0.225)
        elif c == 1: # orange
            return (1, 0.5, 0)
        elif c == 2: # yellow
            return (0.975, 0.99, 0.01)
        elif c == 3: # light green
            return (0.635, 0.97, 0.01)
        elif c == 4: # dark green
            return (0.15, 0.6, 0.07)
        elif c == 5: # light blue
            return (0, 1, 1)
        elif c == 6: # blue
            return (0, 0.02, 0.875)
        else:        # purple
            return (0.6, 0.15, 1)
        
    def init_graphics(self):
        self.light_grey = gtk.gdk.Color(58112,58112,58112)
        self.dark_grey = gtk.gdk.Color(40960,40960,40960)
        self.white = gtk.gdk.color_parse('white')
        self.black = gtk.gdk.color_parse('black')
        self.blue = gtk.gdk.Color(0,1280,57600)
        self.green = gtk.gdk.Color(9728,38912,4608)
        self.red = gtk.gdk.Color(65535,15000,15000)
        self.yellow = gtk.gdk.Color(64000, 65000, 500)
        self.purple = gtk.gdk.color_parse('purple')
        self.orange = gtk.gdk.color_parse('orange')
        self.lblue = gtk.gdk.Color(0, 65535, 65535)
        self.grass = gtk.gdk.Color(41728, 63744, 512)

        self.color_list = [self.red,self.orange,self.yellow,self.grass,self.green,self.lblue,self.blue,self.purple]

    def request_tempo_change(self):
        self.main.score.tempoChange(self.sval)

    def leave_notify_event(self, widget, event):
        if _showDebugMsg:
            print "toolbar.leave"
        return

    def button_press_event(self, widget, event):
        if _showDebugMsg:
            print "toolbar.mouse_btn_press (%d,%d)" % (event.x, event.y)
        self.fix_button = self.fix_button + 1
        if self.fix_button != 1:
            return
        if event.button == 1:
            t = self.toolbar_sel(event.x, event.y)
            if t == -1:
                t = self.is_dragging_slider(event.x, event.y)
                if t != -1:
                    self.is_dragging = 1
                    if self.update_slider_val(event.x):
                        self.request_tempo_change()
                return                    
            elif t < 6:
                flag = 0
                if self.toolsel == t - 1:
                    if self.toolsel == -1:
                        self.pstate = (self.pstate + 1)%3
                        flag = 1
                    elif self.toolsel == 0:
                        self.fstate = (self.fstate + 1)%3
                        flag = 1
                else:
                    self.toolsel = t - 1
                    flag = 1
                    if self.toolsel != 1 and self.main.score.select_state == 1:
                        self.main.score.de_selection()
                        self.main.canvas.draw_deselection()
                if flag == 1:
                    cr = self.pixmap.cairo_create()
                    self.update_toolsel(widget, cr)
                    self.gupdate(widget)
            else:
                self.colorsel = t-6
                self.toolsel = 2
                
                self.update()
                
                self.main.canvas.update_top()
                self.main.score.note_on(self.main.csound, self.colorsel+1, 60, 1)
                
    def update(self):
        cr = self.pixmap.cairo_create()
        widget = self.widget
        self.update_toolsel(widget, cr)
        widget.window.draw_drawable(widget.get_style().fg_gc[gtk.STATE_NORMAL], self.pixmap, 0, 0, 0, 0, self.width, self.height)        

    def button_release_event(self, widget, event):
        if _showDebugMsg:
            print "toolbar.mouse_btn_release (%d,%d)" % (event.x, event.y)
        self.fix_button = 0
        if self.is_dragging == 1:
            self.is_dragging = 0
            if self.update_slider_val(event.x):
                self.request_tempo_change()
            self.deactiviate_slider_btn()
        return

    def configure_event(self, widget, event):
        x, y, width, height = widget.get_allocation()
        self.pixmap = gtk.gdk.Pixmap(widget.window, width, height)
        return True

    def motion_notify_event(self, widget, event):
#        print "toolbar.mouse_move (%d,%d)" % (event.x, event.y)
        if self.is_dragging == 1:
            if self.update_slider_val(event.x):
                self.request_tempo_change()
        return

    def expose_event(self, widget, event):
        cr = self.pixmap.cairo_create()
        self.widget = widget

        # set a clip region for the expose event
        x , y, width, height = event.area
        cr.rectangle(x, y, width, height)
        cr.clip()
        self.draw(widget, cr)

        widget.window.draw_drawable(widget.get_style().fg_gc[gtk.STATE_NORMAL], self.pixmap, x, y, x, y, width, height)

        return False
    
    def gupdate(self, widget):
        widget.window.draw_drawable(widget.get_style().fg_gc[gtk.STATE_NORMAL], self.pixmap, 0, 0, 0, 0, self.width, self.height)        

    def draw_toolbar_frame(self, widget, x, y):

        self.pixmap.draw_rectangle(widget.get_style().white_gc,False,x,y,self.toolbarw,self.toolbarw)
        gc = self.pixmap.new_gc()
        gc.set_rgb_fg_color(self.dark_grey)
        self.pixmap.draw_line(gc, x+1, y+1, x+self.toolbarw-1, y+1)
        self.pixmap.draw_line(gc, x+1, y+1, x+1, y+self.toolbarw-1)
        gc.set_rgb_fg_color(self.light_grey)
        self.pixmap.draw_line(gc, x+self.toolbarw-1, y+2, x+self.toolbarw-1, y+self.toolbarw-1)
        self.pixmap.draw_line(gc, x+2, y+self.toolbarw-1, x+self.toolbarw-1, y+self.toolbarw-1)
        self.pixmap.draw_line(widget.get_style().black_gc, x+2, y+2, x+self.toolbarw-2, y+2)
        self.pixmap.draw_line(widget.get_style().black_gc, x+2, y+2, x+2, y+self.toolbarw-2)

    def load_icons(self, cr):
        self.iconff = cairo.ImageSurface.create_from_png("icon/forte.png");
        self.iconpp = cairo.ImageSurface.create_from_png("icon/piano.png");
        self.iconf = cairo.ImageSurface.create_from_png("icon/frame.png");
        self.icone = cairo.ImageSurface.create_from_png("icon/eraser.png");
        self.iconp = cairo.ImageSurface.create_from_png("icon/pencil.png");
        self.iconh = cairo.ImageSurface.create_from_png("icon/hand.png");
        self.icons = cairo.ImageSurface.create_from_png("icon/scissor.png");
        self.iconcresc = cairo.ImageSurface.create_from_png("icon/cresc.png");
        self.icondim = cairo.ImageSurface.create_from_png("icon/dim.png");
        self.iconfff = cairo.ImageSurface.create_from_png("icon/fortef.png");
        self.iconppf = cairo.ImageSurface.create_from_png("icon/pianof.png");
        
        self.iconlist = [self.iconf, self.iconp, self.icone, self.icons]        
        self.piconlist = [self.iconpp, self.iconppf, self.icondim]
        self.ficonlist = [self.iconff, self.iconfff, self.iconcresc]

        self.load_icon = 1

    def draw_toolbar_icon(self, widget, cr, x, y, img):
        self.draw_toolbar_frame(widget, x, y)
        cr.set_source_surface(img, x+3, y+3)
        cr.paint()        

    def draw_toolbar_colorgrid(self, widget, cr, x, y, color, icon = 0):
        if icon == 1:
            if self.toolsel >= 1:
                self.draw_toolbar_icon(widget, cr, x, y, self.iconlist[self.toolsel-1])
            elif self.toolsel == 0:
                self.draw_toolbar_icon(widget, cr, x, y, self.ficonlist[self.fstate])
            else:
                self.draw_toolbar_icon(widget, cr, x, y, self.piconlist[self.pstate])
        else:
            self.draw_toolbar_frame(widget, x, y)

        c1, c2, c3 = self.get_color_attr(color)
        if icon == 1:
            cr.set_source_rgba(c1, c2, c3, 0.6)
        else:
            cr.set_source_rgb(c1, c2, c3)
        cr.rectangle(x+3, y+3, self.toolbarw-4, self.toolbarw-4)
        cr.fill()

    def draw_slider_btn(self, cr, x, y, len, v):
        xc = v * (len-10) + x + 5
        if self.is_dragging:
            cr.set_source_rgb(0.95, 0.33, 0.33)
        else:
            cr.set_source_rgb(0.9, 0.9, 0.9)
        cr.rectangle(xc - 5, y - 8, 10, 20)
        cr.fill_preserve()
        
        cr.set_source_rgb(0.2, 0.7, 0.1)
        cr.set_line_width(1.2)
        cr.stroke()
        
    def draw_slider(self, cr, x, y, len, v):
        cr.set_source_rgb(0.84, 0.84, 1)
        cr.rectangle(x-2, y-10, len + 4, 24)
        cr.fill()
        
        cr.set_source_rgb(0.7, 0.7, 0.7)
        cr.rectangle(x, y, len, 5)
        cr.fill_preserve()
        
        cr.set_source_rgb(0, 0, 0)
        cr.set_line_width(0.5)
        cr.stroke()
        
        self.draw_slider_btn(cr, x, y, len, v)
        
    def is_dragging_slider(self, x, y):
        x0 = self.sval * (self.sw-10) + self.sx
        y0 = self.sy - 8
        x1 = x0 + 10
        y1 = y0 + 20
        
        if y0 > y or y >= y1 or self.sx > x or x >= self.sx + self.sw:
            return -1
        if y0 <= y and y < y1 and x0 <= x and x < x1:
            return 1
        return 0
    
    def deactiviate_slider_btn(self):
        cr = self.pixmap.cairo_create()
        self.draw_slider(cr, self.sx, self.sy, self.sw, self.sval)
        self.gupdate(self.widget)
        
    def change_slider_val(self, d):
        ov = self.sval
        self.sval = self.sval + d
        if self.sval < 0:
            self.sval = 0
        elif self.sval > 1:
            self.sval = 1
        if ov == self.sval:
            return False
        
        if _showDebugMsg:
            print "slider value = %.2f" % self.sval        
        cr = self.pixmap.cairo_create()
        self.draw_slider(cr, self.sx, self.sy, self.sw, self.sval)
        self.gupdate(self.widget)
        return True
    
    def update_slider_val(self, x):
        x0 = self.sx + 5
        x1 = self.sx + self.sw - 5
        ov = self.sval
        if x > x1:
            self.sval = 1
        elif x < x0:
            self.sval = 0
        else:
            self.sval = 1.0*(x - x0)/(x1 - x0)
        if ov == self.sval:
            return False
        
        if _showDebugMsg:
            print "slider value = %.2f" % self.sval        
        cr = self.pixmap.cairo_create()
        self.draw_slider(cr, self.sx, self.sy, self.sw, self.sval)
        self.gupdate(self.widget)
        return True

    def toolbar_sel(self, x, y):
        if y < self.cy or y >= self.cy + self.toolbarw:
            return -1
        if x >= self.cx and x < self.cx + self.toolbarw*6:
            return (int)((x-self.cx)/self.toolbarw)
        if x < self.cx + self.toolbarw*6 + 16:
            return -1
        t = (int)((x-self.cx-self.toolbarw*6-16)/self.toolbarw)
        if t >= 8:
            return -1
        return t+6
    
    def update_toolsel(self, widget, cr):
        for i in range(8):
            self.draw_toolbar_colorgrid(widget, cr, self.cx + self.toolbarw*(i+6) + 16, self.cy, i, (i == self.colorsel and self.toolsel == 2))
            
        for i in range(6):
            if i == 0:
                self.draw_toolbar_icon(widget, cr, self.cx + self.toolbarw*i, self.cy, self.piconlist[self.pstate])
            elif i == 1:
                self.draw_toolbar_icon(widget, cr, self.cx + self.toolbarw*i, self.cy, self.ficonlist[self.fstate])
            else:
                self.draw_toolbar_icon(widget, cr, self.cx + self.toolbarw*i, self.cy, self.iconlist[i-2])
                
        c1, c2, c3 = self.get_color_attr(self.colorsel)
        cr.set_source_rgba(c1, c2, c3, 0.4)
        cr.rectangle(self.cx + self.toolbarw*(self.toolsel+1) +3, self.cy +3, self.toolbarw-4, self.toolbarw-4)
        cr.fill()

    def draw(self, widget, cr):
        cr.set_source_rgb(0.84,0.84,1)
        cr.rectangle(0, 0, self.width, self.height)
        cr.fill()

        if self.load_icon == 0:
            self.load_icons(cr)
        self.update_toolsel(widget, cr)
        
        self.draw_slider(cr, self.sx, self.sy, self.sw, self.sval)
